//--------------------------------------------------------------------------------------
// File: SimpleSample.fx
//
// The effect file for the SimpleSample sample.  
// 
// Copyright (c) Microsoft Corporation. All rights reserved.
//--------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------
cbuffer lightBuffer
{
	float3 mLightPos;
	
	// transform light source
	float4x4 mLightViewProj;
}

cbuffer objectBuffer
{
	float mAmbientKoeff;
	float mDiffuseKoeff;
	float mSpecularKoeff;
	float mSpecularPower;
	float mDepthBias;
}
	
float4x4 mWorld;                  // World matrix for object
float4x4 mWorldViewProjection;    // World * View * Projection matrix
float4 vecEye;

Texture2D ShadowMap;


RasterizerState backCullRS
{
    CullMode = Back;
    FillMode = Solid;
    MultisampleEnable = false;
};
//--------------------------------------------------------------------------------------
// Texture samplers
//--------------------------------------------------------------------------------------
SamplerState ShadowMapSamp
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};


//--------------------------------------------------------------------------------------
// Vertex shader output structure
//--------------------------------------------------------------------------------------
struct VS_OUTPUT
{
    float4 Position   : SV_Position;   // vertex position 
    float2 TextureUV  : TEXCOORD0;  // vertex texture coords 
    float3 Light	  : TEXCOORD1;     
    float3 View       : TEXCOORD2;
    float3 NormalV	  : TEXCOORD3;
	float4 ShadowMapUV	: TEXCOORD4;
	float Depth			: TEXCOORD5;
};

struct VS_NORMALOUTPUT
{
	float4 Position   : SV_Position;   // vertex position 
    float3 Normal   : NORMAL;   // vertex normal
};

struct VS_SIMPLE_OUTPUT
{
    float4 Position   	: SV_Position;   // vertex position 
    float4 WorldPos	  	: TEXCOORD0;     
};


//--------------------------------------------------------------------------------------
// This shader computes standard transform and lighting
//--------------------------------------------------------------------------------------
VS_NORMALOUTPUT RenderNormalsVS( float4 vPos : POSITION, 
                         float3 vNormal : NORMAL,
                         float2 vTexCoord0 : TEXCOORD0 )
{
    VS_NORMALOUTPUT Output;
    
    // Transform the position from object space to homogeneous projection space
    Output.Position = mul(vPos, mWorldViewProjection);
    
    // Just copy the normals through
    Output.Normal = vNormal; 
    
    return Output;    
}

//--------------------------------------------------------------------------------------
// This shader computes standard transform and lighting
//--------------------------------------------------------------------------------------
VS_OUTPUT RenderSceneVS( float4 vPos : POSITION, 
                         float3 vNormal : NORMAL,
                         float2 vTexCoord0 : TEXCOORD0 )
{
    VS_OUTPUT Output;
    float3 vNormalWorldSpace;
    
    // Transform the position from object space to homogeneous projection space
    Output.Position = mul(vPos, mWorldViewProjection);
    
    // Just copy the texture coordinate through
    Output.TextureUV = vTexCoord0; 
    
    // Eye Vector in World Coordinates -> transform into Tangent Space
    float4 PosWorld = mul(vPos, mWorld);
    Output.View = vecEye - PosWorld;
    
   	Output.Light = mLightPos.xyz - PosWorld.xyz;
    
    Output.NormalV = normalize(mul(vNormal, mWorld));
    
	
	// measure distance between light and point
	float4 LightPosition = mul(PosWorld, mLightViewProj);
	Output.Depth = length( mLightPos.xyz - PosWorld.xyz )- mDepthBias;	
    
    // transform projected textures
	Output.ShadowMapUV = LightPosition;
    
    return Output;    
}



VS_SIMPLE_OUTPUT BlackSceneVS( float4 vPos : POSITION, 
                         float3 vNormal : NORMAL,
                         float2 vTexCoord0 : TEXCOORD0 )
{
    VS_SIMPLE_OUTPUT Output;
    Output.Position = mul(vPos, mWorldViewProjection);
    float4 PosWorld = mul(vPos, mWorld);
    Output.WorldPos = PosWorld;
    
    return Output;    
}

float VSM_FILTER( float2 tex, float fragDepth )
{
    float lit = (float)0.0f;
    float2 moments = ShadowMap.SampleLevel( ShadowMapSamp, tex, 0 );
    
    float E_x2 = moments.y;
    float Ex_2 = moments.x * moments.x;
    float variance = E_x2 - Ex_2;    
    float mD = (moments.x - fragDepth );
    float mD_2 = mD * mD;
    float p = 0.0f;
    if (variance > 0.0f)
    {
    	p = variance / (variance + mD_2 );
    }
    lit = max( p, fragDepth <= moments.x );
    
    return lit;
}

//--------------------------------------------------------------------------------------
// This shader outputs the pixel's color by modulating the texture's
// color with diffuse material color
//--------------------------------------------------------------------------------------
float4 RenderScenePS( VS_OUTPUT In ) : SV_Target
{ 
        // Get the texture color
    float4 Color = float4(1.0f, 1.0f, 1.0f, 1.0f);
    
    // Fetch Shadow Map:
    float2 tex = (In.ShadowMapUV.xy/In.ShadowMapUV.w) * float2( 0.5f, -0.5f ) + 0.5f;
    float lit = VSM_FILTER( tex, In.Depth );
    
    /*float4 returnvalue;
    returnvalue.x = In.NormalV.x;
    returnvalue.y = In.NormalV.y;
    returnvalue.z = In.NormalV.z;
    returnvalue.w = 1.0f;
    return returnvalue*/;
    
    float4 OutputColor = mAmbientKoeff * Color;
    
    if (lit > 0.0f )
    {

		//get Normal Map 
		float3 Normal = normalize(In.NormalV);
	    
		float3 LightDir = normalize(In.Light);
		float3 ViewDir = normalize(In.View);
	    
		//Diffuse Component:
		float Diffuse = saturate(dot(Normal, LightDir));
	    
		//Self-Shadowing:
		float Shadow = saturate(4.0f * Diffuse);
	   
		if (Shadow > 0.0f)
		{ 
	    
			//Reflection Vector:
			float3 Reflect = normalize(2 * Diffuse * Normal - LightDir);
		    	
			//Specular Reflection:
			float Specular = saturate(pow(saturate(dot(Reflect, ViewDir)), mSpecularPower));		    	
		    	
			OutputColor += ((mDiffuseKoeff * Color * Diffuse) + (mSpecularKoeff * Color * Specular * saturate(Shadow))) * lit;
		}
		else
		{
    		OutputColor += (mDiffuseKoeff * Color*Diffuse) * lit;
		}
	}
    
    return OutputColor;
}

//--------------------------------------------------------------------------------------
// This shader outputs the Normal as the Pixels Color
//--------------------------------------------------------------------------------------
float4 RenderNormalsPS( VS_NORMALOUTPUT In ) : SV_Target
{ 
	float4 color;
	color.rgb = saturate(normalize(In.Normal.rgb));
	color.a = 1.0f;
    return color;
}

float2 BlackScenePS( VS_SIMPLE_OUTPUT In ) : SV_Target
{ 
    
    float2 OutputColor = {0.0, 0.0};
	OutputColor.y = length(vecEye - In.WorldPos);
	
    return OutputColor;
}


//--------------------------------------------------------------------------------------
// Renders scene 
//--------------------------------------------------------------------------------------
technique10 RenderScene
{
    pass P0
    {       
		SetVertexShader( CompileShader( vs_4_0, RenderSceneVS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, RenderScenePS() ) );
    }
}


technique10 Normals
{
    pass P0
    {       
		SetVertexShader( CompileShader( vs_4_0, RenderNormalsVS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, RenderNormalsPS() ) );
    }
}

technique10 RenderBlackScene
{
    pass P0
    {      
    	SetRasterizerState( backCullRS ); 
		SetVertexShader( CompileShader( vs_4_0, BlackSceneVS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, BlackScenePS() ) );
    }
}