//--------------------------------------------------------------------------------------
// File: SimpleSample.fx
//
// The effect file for the SimpleSample sample.  
// 
// Copyright (c) Microsoft Corporation. All rights reserved.
//--------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------
cbuffer cb0
{
	float4x4 g_mWorldViewProjection;    // World * View * Projection matrix
	float4x4 g_mWorld;                  // World matrix for object
	float4x4 g_mProj;
	float4x4 g_mViewCM[6]; // View matrices for cube map rendering
	float3 g_mLightPos;
}

//--------------------------------------------------------------------------------------
// Cubemap via Geometry Shader
//--------------------------------------------------------------------------------------

struct VS_CUBEMAP_IN
{
	float4 Pos		: POSITION;
};

struct GS_CUBEMAP_IN
{
	float4 Pos		: SV_POSITION;    // World position
    float Depth		: TEXCOORD0;         // Texture coord
};


struct PS_CUBEMAP_IN
{
    float4 Pos : SV_POSITION;     // Projection coord
    float Depth : TEXCOORD0;       // Texture coord
    uint RTIndex : SV_RenderTargetArrayIndex;
};

GS_CUBEMAP_IN VS_CubeMap( VS_CUBEMAP_IN input )
{
    GS_CUBEMAP_IN output = (GS_CUBEMAP_IN)0.0f;

    float4 worldPos = mul(input.Pos, g_mWorld);
    output.Depth = length( g_mLightPos.xyz - worldPos.xyz );

    // Compute world position
    output.Pos = worldPos;

    return output;
}

[maxvertexcount(18)]
void GS_CubeMap( triangle GS_CUBEMAP_IN input[3], inout TriangleStream<PS_CUBEMAP_IN> CubeMapStream )
{
    for( int f = 0; f < 6; ++f )
    {
        // Compute screen coordinates
        PS_CUBEMAP_IN output;
        output.RTIndex = f;
        for( int v = 0; v < 3; v++ )
        {
            output.Pos = mul( input[v].Pos, g_mViewCM[f] );
            output.Pos = mul( output.Pos, g_mProj );
            output.Depth = input[v].Depth;
            CubeMapStream.Append( output );
        }
        CubeMapStream.RestartStrip();
    }
}


float4 PS_CubeMap( PS_CUBEMAP_IN input ) : SV_Target
{
    return float4( input.Depth, input.Depth * input.Depth, 0.0f, 1.0f );
}


//--------------------------------------------------------------------------------------
// Vertex shader output structure
//--------------------------------------------------------------------------------------
struct VS_OUTPUT
{
    float4 Position   : SV_Position;   // vertex position 
    float Depth  : TEXCOORD0;  
};

//--------------------------------------------------------------------------------------
// This shader computes standard transform and lighting
//--------------------------------------------------------------------------------------
VS_OUTPUT CreateShadowMapVS( float4 vPos : POSITION )
{
    VS_OUTPUT Output;
    
    // Transform the position from object space to homogeneous projection space
    float4 Pos = mul(vPos, g_mWorldViewProjection);
    Output.Position = Pos;
    
    float4 worldPos = mul(vPos, g_mWorld);
    Output.Depth = length( g_mLightPos.xyz - worldPos.xyz );
    
    return Output;    
}

//--------------------------------------------------------------------------------------
// This shader outputs the pixel's color by modulating the texture's
// color with diffuse material color
//--------------------------------------------------------------------------------------
float4 CreateShadowMapPS( VS_OUTPUT In ) : SV_Target
{ 
    //return In.Depth;    
    return float4( In.Depth, In.Depth * In.Depth, 0.0f, 1.0f );
}


//--------------------------------------------------------------------------------------
// Renders scene 
//--------------------------------------------------------------------------------------
technique10 CreateShadowMap
{
    pass P0
    {       
    	SetVertexShader( CompileShader( vs_4_0, CreateShadowMapVS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, CreateShadowMapPS() ) );
    }
}

technique10 CreateCubeShadowMap
{
    pass p0
    {
        SetVertexShader( CompileShader( vs_4_0, VS_CubeMap() ) );
        SetGeometryShader( CompileShader( gs_4_0, GS_CubeMap() ) );
        SetPixelShader( CompileShader( ps_4_0, PS_CubeMap() ) );
    }
};