BlendState NoBlending
{
    BlendEnable[0] = FALSE;
};

BlendState SrcAlphaBlendingAdd
{
    BlendEnable[0] = TRUE;
    SrcBlend = SRC_ALPHA;
    DestBlend = INV_SRC_ALPHA;
    BlendOp = ADD;
    SrcBlendAlpha = ZERO;
    DestBlendAlpha = ZERO;
    BlendOpAlpha = ADD;
    RenderTargetWriteMask[0] = 0x0F;
};

BlendState SrcColorBlendingAdd
{
    BlendEnable[0] = TRUE;
    SrcBlend = SRC_COLOR;
    DestBlend = ONE;
    BlendOp = ADD;
    SrcBlendAlpha = ZERO;
    DestBlendAlpha = ZERO;
    BlendOpAlpha = ADD;
    RenderTargetWriteMask[0] = 0x0F;
};



//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------
cbuffer sunBuffer
{
	float3 mSunPos;
		
	// transform light source
	float4x4 mSunViewProj;
	float mSunIntensity;
}
Texture2D SunShadowMap;

cbuffer cubeLightBuffer
{
	float3 mCubeLightPos;
	float mCubeLightIntensity;
	float mCubeLightFalloff;
	bool mUseCubeLight;
}
TextureCube CubeLightShadowMap;

cbuffer objectBuffer2
{
	float mAmbientKoeff;
}

cbuffer objectBuffer
{
	float mDiffuseKoeff;
	float mSpecularKoeff;
	float mSpecularWhiteKoeff;
	float mSpecularPower;
	float mDepthBias;
	float mUseAlpha;
	bool useReflectionSurface;
}
	

float4x4 mWorld;                  // World matrix for object
float4x4 mWorldViewProjection;    // World * View * Projection matrix
float4x4 mViewProjection;
float4 vecEye;
bool useRenderThreshold;
float renderThreshold;
float reflTexOffset;

// Environment Mapping:
float4x4 mCubeView[6]; // View matrices for cube map rendering
float4x4 mCubeProj;

Texture2D g_MeshTexture;              // Color texture for mesh
Texture2D g_MeshTexture_normal;       // Normal texture for mesh
Texture2D g_MeshTexture_reflection;       // Reflection texture for mesh


RasterizerState backCullRS
{
    CullMode = Back;
    FillMode = Solid;
    MultisampleEnable = false;
};

//--------------------------------------------------------------------------------------
// Texture samplers
//--------------------------------------------------------------------------------------
SamplerState MeshTextureSampler
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};

SamplerState SunShadowMapSamp
{
    Filter = MIN_MAG_MIP_LINEAR; //ANISOTROPIC;
    AddressU = Clamp;
    AddressV = Clamp;
};

SamplerState CubeMapSampler
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

// GPU GEMS 3 Light Bleeding reduction:
float linstep(float min, float max, float v)
{
	return clamp((v-min) / (max-min), 0, 1);
}

float ReduceLightBleeding(float p, float amount)
{
	return linstep(amount, 1, p);
}

float VarianceShadowMap( float2 tex, float depth )
{
    float2 moments = SunShadowMap.Sample( SunShadowMapSamp, tex ); 

    float variance = moments.y - (moments.x * moments.x);   
    float d = (moments.x - depth ) * (moments.x - depth );
    
    float p = 0.0f;
    
    if (variance > 0.0f)
    {
    	p = ReduceLightBleeding(variance / (variance + d ), 0.6f);
    }
    
   return max( p, depth <= moments.x );
}

float CubeVarianceShadowMap( float3 tex, float depth )
{
	float texOffset = 4.0f/512.0f;
	
    float2 moments = CubeLightShadowMap.Sample( CubeMapSampler, tex ); 
    
    
    float3 texNeu = tex;
    texNeu.x += texOffset;
    moments += CubeLightShadowMap.Sample( CubeMapSampler, texNeu );
    texNeu = tex;
    texNeu.x -= texOffset;
    moments += CubeLightShadowMap.Sample( CubeMapSampler, texNeu );
    texNeu = tex;
    texNeu.y += texOffset;
    moments += CubeLightShadowMap.Sample( CubeMapSampler, texNeu );
    texNeu = tex;
    texNeu.y -= texOffset;
    moments += CubeLightShadowMap.Sample( CubeMapSampler, texNeu );
    texNeu = tex;
    texNeu.z += texOffset;
    moments += CubeLightShadowMap.Sample( CubeMapSampler, texNeu );
    texNeu = tex;
    texNeu.z -= texOffset;
    moments += CubeLightShadowMap.Sample( CubeMapSampler, texNeu );
    
    moments /= 7.0f;
    

    float variance = moments.y - (moments.x * moments.x);   
    float d = (moments.x - depth ) * (moments.x - depth );
    
    float p = 0.0f;
    
    if (variance > 0.0f)
    {
    	p = ReduceLightBleeding(variance / (variance + d ), 0.4f);
    }
    
   return max( p, depth <= moments.x );
}

//--------------------------------------------------------------------------------------
// Cubemap via Geometry Shader
//--------------------------------------------------------------------------------------

struct VS_CUBEMAP_IN
{
	float4 vPos			: POSITION;
	float3 vNormal  	: NORMAL;
    float2 vTexCoord0 	: TEXCOORD0;
};


struct PS_CUBEMAP_IN
{
    float4 Position     : SV_POSITION;     // Projection coord
    float2 TextureUV  	: TEXCOORD0;  // vertex texture coords 
    float4 WorldPos	  	: TEXCOORD1;     
    float3 View     	: TEXCOORD2;
	float4 ShadowMapUV	: TEXCOORD3;
	float3 WorldNorm 	: TEXCOORD4;
};

PS_CUBEMAP_IN VS_CubeMap( VS_CUBEMAP_IN input )
{
    PS_CUBEMAP_IN Output = (PS_CUBEMAP_IN)0.0f;

    float4 PosWorld = mul(input.vPos, mWorld);
    Output.View = vecEye - PosWorld;
    Output.WorldPos = PosWorld;
    Output.Position = PosWorld;
    
    // Just copy the texture coordinate through
    Output.TextureUV = input.vTexCoord0; 

    Output.WorldNorm = mul(input.vNormal, mWorld);
        
    // transform projected textures
	Output.ShadowMapUV = mul(PosWorld, mSunViewProj);;

    return Output;
}




float4 PS_CubeMap( PS_CUBEMAP_IN In ) : SV_Target
{
    // Get the texture color
    float4 Color = g_MeshTexture.Sample(MeshTextureSampler, In.TextureUV);
    float4 OutputColor = mAmbientKoeff * Color;

	float3 Normal = normalize(In.WorldNorm);
	
	float3 ViewDir = normalize(In.View);
	
	/*float3 debugcolor = Normal;//normalize(abs(In.WorldBiNorm));
	return float4(debugcolor.x, debugcolor.y, debugcolor.z, 1.0f);*/
	
	// *************************
	// Sun Lighting Calculation:
	// *************************
		
	float3 SunDir = mSunPos.xyz - In.WorldPos.xyz;
	float3 SunDirNorm = normalize(SunDir);
	    
	//Diffuse Component:
	float SunDiffuse = saturate(dot(Normal, SunDirNorm));
	    
	OutputColor += (mDiffuseKoeff * Color * SunDiffuse) * mSunIntensity;
	
	// ****************************
	// End Sun Lighting Calculation
	// ****************************
	
    
    OutputColor.a = Color.a;
    
    return OutputColor;
}

//--------------------------------------------------------------------------------------
// Vertex shader output structure
//--------------------------------------------------------------------------------------
struct VS_OUTPUT
{
    float4 Position   	: SV_Position;   // vertex position 
    float2 TextureUV  	: TEXCOORD0;  // vertex texture coords 
    float4 WorldPos	  	: TEXCOORD1;     
    float3 View     	: TEXCOORD2;
	float4 ShadowMapUV	: TEXCOORD3;
	float3 SunDir	 	: TEXCOORD4;
	float3 CubeLightDir	: TEXCOORD5;
	float4 SSPos		: TEXCOORD6;
};

struct VS_OUTPUT_WORLD
{
    float4 Position   	: SV_Position;   // vertex position 
    float2 TextureUV  	: TEXCOORD0;  // vertex texture coords 
    float4 WorldPos	  	: TEXCOORD1;     
    float3 View     	: TEXCOORD2;
	float4 ShadowMapUV	: TEXCOORD3;
	float3 WorldNorm 	: TEXCOORD4;
	float3 WorldBiNorm 	: TEXCOORD5;
	float3 WorldTangent : TEXCOORD6;
	float4 SSPos		: TEXCOORD7;
};

struct VS_SIMPLE_OUTPUT
{
    float4 Position   	: SV_Position;   // vertex position 
    float2 TextureUV  	: TEXCOORD0;  // vertex texture coords 
    float4 WorldPos	  	: TEXCOORD1;     
};


//--------------------------------------------------------------------------------------
// This shader computes standard transform and lighting
//--------------------------------------------------------------------------------------
VS_OUTPUT RenderSceneVS( float4 vPos : POSITION, 
                         float3 vNormal : NORMAL,
                         float2 vTexCoord0 : TEXCOORD0,
                         float4 vTangent : TANGENT )
{
    VS_OUTPUT Output;
    
    // Transform the position from object space to homogeneous projection space
    Output.Position = mul(vPos, mWorldViewProjection);
    Output.SSPos = Output.Position;
    
    // Just copy the texture coordinate through
    Output.TextureUV = vTexCoord0; 

    // Light is in World Space -> transform into Tangent Space
    float3 tangent = vTangent.xyz ;
    float3 bitangent = cross(tangent, vNormal);
    
    float3x3 worldToTangentSpace;
    worldToTangentSpace[0] = normalize(mul(tangent, mWorld));
    worldToTangentSpace[1] = normalize(mul(bitangent, mWorld));
    worldToTangentSpace[2] = normalize(mul(vNormal, mWorld));
    
    // Eye Vector in World Coordinates -> transform into Tangent Space
    float4 PosWorld = mul(vPos, mWorld);
    Output.View = mul(worldToTangentSpace, vecEye - PosWorld);
    Output.WorldPos = PosWorld;
    
    Output.SunDir = mul(worldToTangentSpace, mSunPos - PosWorld.xyz);
    Output.CubeLightDir = mul(worldToTangentSpace, mCubeLightPos - PosWorld.xyz);
	
	// measure distance between light and point
	float4 LightPosition = mul(PosWorld, mSunViewProj);
    
    // transform projected textures
	Output.ShadowMapUV = LightPosition;
    
    return Output;    
}

//--------------------------------------------------------------------------------------
// This shader computes standard transform and lighting
//--------------------------------------------------------------------------------------
VS_OUTPUT_WORLD RenderSceneWorldSpaceVS( float4 vPos : POSITION, 
                         float3 vNormal : NORMAL,
                         float2 vTexCoord0 : TEXCOORD0,
                         float4 vTangent : TANGENT )
{
    VS_OUTPUT_WORLD Output;
    
    // Transform the position from object space to homogeneous projection space
    Output.Position = mul(vPos, mWorldViewProjection);
    Output.SSPos = Output.Position;
    
    // Just copy the texture coordinate through
    Output.TextureUV = vTexCoord0; 

    // Light is in World Space -> transform into Tangent Space
    float3 tangent = vTangent.xyz ;
    float3 bitangent = cross(vNormal, tangent);
    
    Output.WorldTangent = mul(tangent, mWorld);
    Output.WorldBiNorm = mul(bitangent, mWorld);
    Output.WorldNorm = mul(vNormal, mWorld);
    
    // Eye Vector in World Coordinates -> transform into Tangent Space
    float4 PosWorld = mul(vPos, mWorld);
    Output.View = vecEye - PosWorld;
    Output.WorldPos = PosWorld;
	
	// measure distance between light and point
	float4 LightPosition = mul(PosWorld, mSunViewProj);
    
    // transform projected textures
	Output.ShadowMapUV = LightPosition;
    
    return Output;    
}

VS_SIMPLE_OUTPUT BlackSceneVS( float4 vPos : POSITION, 
                         float3 vNormal : NORMAL,
                         float2 vTexCoord0 : TEXCOORD0,
                         float4 vTangent : TANGENT )
{
    VS_SIMPLE_OUTPUT Output;
    Output.Position = mul(vPos, mWorldViewProjection);
    float4 PosWorld = mul(vPos, mWorld);
    Output.WorldPos = PosWorld;
    // Just copy the texture coordinate through
    Output.TextureUV = vTexCoord0; 
    
    return Output;    
}



float2 computeTextureCoord(float4 positionWS, float3 directionWS)
{
	float4 positionSS;

  // Compute the position after traveling a fixed distance
  positionWS.xz += directionWS.xz * 1.0f;
  positionWS.w = 1.0f;

  // Compute the screen space position of the newly computed position
  positionSS = mul(positionWS, mViewProjection);

  // Do the perspective divide
  positionSS.xy /= positionSS.w;

  // Convert screen space position from [-1,-1]->[1,1] to [0,0]->[1,1]
  positionSS.xy = positionSS.xy * 0.5 + 0.5;

  return(positionSS.xy);
}

//--------------------------------------------------------------------------------------
// This shader outputs the pixel's color by modulating the texture's
// color with diffuse material color
//--------------------------------------------------------------------------------------
float4 RenderScenePS( VS_OUTPUT In ) : SV_Target
{ 
	if (useRenderThreshold)
	{
		if(In.WorldPos.y < renderThreshold)
			discard;
	}
	
    // Get the texture color
    float4 Color = g_MeshTexture.Sample(MeshTextureSampler, In.TextureUV + float2(reflTexOffset, reflTexOffset));
    if (mUseAlpha)
    {
    	if (Color.a<0.1f)
    		discard;
    }
	
    
    float4 OutputColor = mAmbientKoeff * Color;
    
   //get Normal Map 
	float3 TSNormal = (2 * (g_MeshTexture_normal.Sample(MeshTextureSampler, In.TextureUV + float2(reflTexOffset, reflTexOffset)))) - 1.0;
	float3 Normal = normalize(TSNormal);
    
	
	float3 ViewDir = normalize(In.View);
	
	/*float3 debugcolor = Normal;//normalize(abs(In.WorldBiNorm));
	return float4(debugcolor.x, debugcolor.y, debugcolor.z, 1.0f);*/
	
	// *************************
	// Sun Lighting Calculation:
	// *************************
		
	float3 SunDir = In.SunDir;
	float3 SunDirNorm = normalize(SunDir);
	float SunDepth = length( SunDir ) - mDepthBias;
	
	// Fetch Shadow Map:
    float2 SunTex = (In.ShadowMapUV.xy/In.ShadowMapUV.w) * float2( 0.5f, -0.5f ) + 0.5f;
    float SunVsm = VarianceShadowMap( SunTex, SunDepth );
    SunVsm *= mSunIntensity;
	    
	//Diffuse Component:
	float SunDiffuse = saturate(dot(Normal, SunDirNorm));
	    
	//Self-Shadowing:
	float SunShadow = saturate(4.0f * SunDir.z);
	  
	if (SunShadow > 0.0f)
	{ 
	    
		//Reflection Vector:
		float3 SunReflect = normalize(2 * SunDiffuse * Normal - SunDirNorm);
		    	
		//Specular Reflection:
		float SunSpecular = pow(saturate(dot(SunReflect, ViewDir)), mSpecularPower);
		SunSpecular *= saturate(SunShadow);
		    	
		    	
		OutputColor += ((mDiffuseKoeff * Color * SunDiffuse) + (mSpecularKoeff * Color * SunSpecular)) * SunVsm;
	}
	else
	{
    	OutputColor += (mDiffuseKoeff * Color * SunDiffuse) * SunVsm;
	}
	
	// ****************************
	// End Sun Lighting Calculation
	// ****************************
	
	// *************************
	// Cube Lighting Calculation:
	// *************************
		
	if (mUseCubeLight)
	{
		float3 CubeLightDir = In.CubeLightDir;
		float3 CubeLightDirNorm = normalize(CubeLightDir);
		float CubeLightDepth = length( CubeLightDir ) - mDepthBias;
		float CubeLightDepthFrag = CubeLightDepth / 2.0f;
		
		// Fetch Shadow Map:
    	float CubeLightVsm = CubeVarianceShadowMap( CubeLightDirNorm * -1.0f, CubeLightDepth );
    	CubeLightVsm *= mCubeLightIntensity * pow(mCubeLightFalloff, CubeLightDepthFrag * CubeLightDepthFrag);
    	
	    	
		//Diffuse Component:
		float CubeLightDiffuse = saturate(dot(Normal, CubeLightDirNorm));
	    	
		//Self-Shadowing:
		float CubeLightShadow = saturate(4.0f * CubeLightDir);
	  			
	  	if (CubeLightShadow > 0.0f)
		{ 
	    	
			//Reflection Vector:
			float3 CubeLightReflect = normalize(2 * CubeLightDiffuse * Normal - CubeLightDirNorm);
		    		
			//Specular Reflection:
			float CubeLightSpecular = pow(saturate(dot(CubeLightReflect, ViewDir)), mSpecularPower);
			CubeLightSpecular *= saturate(CubeLightShadow);
		    		
		    		
			OutputColor += ((mDiffuseKoeff * Color * CubeLightDiffuse) + (mSpecularKoeff * Color * CubeLightSpecular)) * CubeLightVsm;
		}
		else
		{
    		OutputColor += (mDiffuseKoeff * Color * CubeLightDiffuse) * CubeLightVsm;
		}
	}
	
	// ****************************
	// End Cube Lighting Calculation
	// ****************************
    
    OutputColor.a = Color.a;
    
    return OutputColor;
}

//--------------------------------------------------------------------------------------
// This shader outputs the pixel's color by modulating the texture's
// color with diffuse material color
//--------------------------------------------------------------------------------------
float4 RenderSceneWorldSpacePS( VS_OUTPUT_WORLD In ) : SV_Target
{ 

	
    // Get the texture color
    float4 Color = g_MeshTexture.Sample(MeshTextureSampler, In.TextureUV + float2(reflTexOffset, reflTexOffset));
    if (mUseAlpha)
    {
    	if (Color.a<0.1f)
    		discard;
    }
	
	float3x3 TangentToWorldSpace;
	TangentToWorldSpace._11_21_31 = In.WorldTangent;
	TangentToWorldSpace._12_22_32 = In.WorldBiNorm;
	TangentToWorldSpace._13_23_33 = In.WorldNorm;
    
    float4 OutputColor = mAmbientKoeff * Color;
    
   //get Normal Map 
	float3 TSNormal = (2 * (g_MeshTexture_normal.Sample(MeshTextureSampler, In.TextureUV + float2(reflTexOffset, reflTexOffset)))) - 1.0;
	float3 Normal = normalize(mul(TangentToWorldSpace, TSNormal));
    
	if (useReflectionSurface)
	{
		//float2 reflectionUV = ( In.SSPos.xy /  In.SSPos.w) * 0.5f + 0.5f;
		
		float2 reflectionUV = computeTextureCoord(In.WorldPos, Normal);
		
		float4 reflectioncolor = g_MeshTexture_reflection.Sample(SunShadowMapSamp, reflectionUV);
		reflectioncolor.a = 1.0f;
		//float4 reflectioncolor = float4(reflectionUV.x, reflectionUV.y, 0.0f, 1.0f);
		//Color = reflectioncolor;
		Color = lerp(reflectioncolor, Color, 0.7f);
	}
	
	float3 ViewDir = normalize(In.View);
	
	/*float3 debugcolor = Normal;//normalize(abs(In.WorldBiNorm));
	return float4(debugcolor.x, debugcolor.y, debugcolor.z, 1.0f);*/
	
	// *************************
	// Sun Lighting Calculation:
	// *************************
		
	float3 SunDir = mSunPos.xyz - In.WorldPos.xyz;
	float3 SunDirNorm = normalize(SunDir);
	float SunDepth = length( SunDir ) - mDepthBias;
	
	// Fetch Shadow Map:
    float2 SunTex = (In.ShadowMapUV.xy/In.ShadowMapUV.w) * float2( 0.5f, -0.5f ) + 0.5f;
    float SunVsm = VarianceShadowMap( SunTex, SunDepth );
    SunVsm *= mSunIntensity;
	    
	//Diffuse Component:
	float SunDiffuse = saturate(dot(Normal, SunDirNorm));
	    
	//Self-Shadowing:
	float SunShadow = saturate(4.0f * SunDiffuse);
	  
	if (SunShadow > 0.0f)
	{ 
	    
		//Reflection Vector:
		float3 SunReflect = normalize(2 * SunDiffuse * Normal - SunDirNorm);
		    	
		//Specular Reflection:
		float SunSpecular = pow(saturate(dot(SunReflect, ViewDir)), mSpecularPower);
		SunSpecular *= saturate(SunShadow);
		    	
		    	
		OutputColor += ((mDiffuseKoeff * Color * SunDiffuse) + (mSpecularKoeff * Color * SunSpecular)) * SunVsm;
	}
	else
	{
    	OutputColor += (mDiffuseKoeff * Color * SunDiffuse) * SunVsm;
	}
	
	// ****************************
	// End Sun Lighting Calculation
	// ****************************
	
	// *************************
	// Cube Lighting Calculation:
	// *************************
		
	if (mUseCubeLight)
	{
		float3 CubeLightDir = mCubeLightPos.xyz - In.WorldPos.xyz;
		float3 CubeLightDirNorm = normalize(CubeLightDir);
		float CubeLightDepth = length( CubeLightDir ) - mDepthBias;
		float CubeLightDepthFrag = CubeLightDepth / 2.0f;
		
		// Fetch Shadow Map:
    	float CubeLightVsm = CubeVarianceShadowMap( CubeLightDirNorm * -1.0f, CubeLightDepth );
    	CubeLightVsm *= mCubeLightIntensity * pow(mCubeLightFalloff, CubeLightDepthFrag * CubeLightDepthFrag);
    	
    	//DEBUG:
    	/*CubeLightVsm = 1.0f;
    	float4 moments = CubeLightShadowMap.Sample( CubeMapSampler, CubeLightDir * -1.0f );
    	if (moments.x > 0)
    		return float4(1.0f, 1.0f, 1.0f, 1.0f);
    	else
    		return float4(0.0f, 0.0f, 0.0f, 1.0f);*/
    	//END DEBUG
	    	
		//Diffuse Component:
		float CubeLightDiffuse = saturate(dot(Normal, CubeLightDirNorm));
	    	
		//Self-Shadowing:
		float CubeLightShadow = saturate(4.0f * CubeLightDiffuse);
	  	
		if (CubeLightShadow > 0.0f)
		{ 
	    	
			//Reflection Vector:
			float3 CubeLightReflect = normalize(2 * CubeLightDiffuse * Normal - CubeLightDirNorm);
		    		
			//Specular Reflection:
			float CubeLightSpecular = pow(saturate(dot(CubeLightReflect, ViewDir)), mSpecularPower);
			CubeLightSpecular *= saturate(CubeLightShadow);
		    		
		    		
			OutputColor += ((mDiffuseKoeff * Color * CubeLightDiffuse) + (mSpecularKoeff * Color * CubeLightSpecular)) * CubeLightVsm;
		}
		else
		{
    		OutputColor += (mDiffuseKoeff * Color * CubeLightDiffuse) * CubeLightVsm;
		}
	}
	
	// ****************************
	// End Cube Lighting Calculation
	// ****************************
    
    OutputColor.a = Color.a;
    
    return OutputColor;
}

float4 RenderTexturePS( VS_OUTPUT In ) : SV_Target
{ 
    // Get the texture color
    return  g_MeshTexture.Sample(MeshTextureSampler, In.TextureUV) * (0.3f + mCubeLightIntensity);
}

float2 BlackScenePS( VS_SIMPLE_OUTPUT In ) : SV_Target
{ 
    if (mUseAlpha)
    {
        float4 Color = g_MeshTexture.Sample(MeshTextureSampler, In.TextureUV + float2(reflTexOffset, reflTexOffset));
    	if (Color.a<0.8f)
    		discard;
    }
    float2 OutputColor = {0.0, 0.0};
	OutputColor.y = length(vecEye - In.WorldPos);
	
    return OutputColor;
}



//--------------------------------------------------------------------------------------
// Renders scene 
//--------------------------------------------------------------------------------------
technique10 RenderScene
{
    pass P0
    {      
    	SetRasterizerState( backCullRS ); 
		SetVertexShader( CompileShader( vs_4_0, RenderSceneVS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, RenderScenePS() ) );
        
        SetBlendState( SrcAlphaBlendingAdd, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
    }
}

technique10 RenderSceneWorldSpace
{
    pass P0
    {      
    	SetRasterizerState( backCullRS ); 
		SetVertexShader( CompileShader( vs_4_0, RenderSceneWorldSpaceVS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, RenderSceneWorldSpacePS() ) );
        
        SetBlendState( SrcAlphaBlendingAdd, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
    }
}

technique10 RenderTexture
{
    pass P0
    {      
    	SetRasterizerState( backCullRS ); 
		SetVertexShader( CompileShader( vs_4_0, RenderSceneVS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, RenderTexturePS() ) );
        
        SetBlendState( SrcAlphaBlendingAdd, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
    }
}

technique10 RenderBlackScene
{
    pass P0
    {      
    	SetBlendState( NoBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
    	
    	SetRasterizerState( backCullRS ); 
		SetVertexShader( CompileShader( vs_4_0, BlackSceneVS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, BlackScenePS() ) );
    }
}


technique10 RenderToEnvironmentMap
{
    pass p0
    {
    	SetRasterizerState( backCullRS ); 
        SetVertexShader( CompileShader( vs_4_0, VS_CubeMap() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS_CubeMap() ) );
    }
};