//#############################################################################
//
// DOWNSAMPLING, BLURRING
//
//#############################################################################

#define MAX_SAMPLES 4

// Contains weights for Gaussian blurring
float g_GWeights[MAX_SAMPLES];

DepthStencilState DisableDepth
{
    DepthEnable    = FALSE;
    DepthWriteMask = ZERO;
};

Texture2D g_SourceTex : TEXTURE0;

SamplerState BilinearSampler
{
    Filter   = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};


struct VS_Input
{
    float2 Pos    : POSITION;
    float4 Tex[2] : TEXCOORD0;
};

struct VS_Output
{
    float4 Pos    : SV_POSITION;
    float2 Tex : TEXCOORD0;
};

struct VS_Blur_Input
{
    float2 Pos                : POSITION;
    float4 Tex[MAX_SAMPLES/2] : TEXCOORD0;
};

struct VS_Blur_Output {
    float4 Pos                : SV_POSITION;
    float4 Tex[MAX_SAMPLES/2] : TEXCOORD0;
};


VS_Output VS_downscale(in float4 Pos : POSITION) 
{
    VS_Output OUT;

    OUT = (VS_Output)0;

    OUT.Pos    = float4(Pos.x, Pos.y, 0.5f, 1.0f);
    OUT.Tex    = Pos.zw;

    return OUT;
}

float4 PS_downScale2x2(in VS_Output IN) : SV_TARGET0
{
    return g_SourceTex.Sample(BilinearSampler, IN.Tex.xy) ;

}


VS_Blur_Output VS_blur(in VS_Blur_Input IN)
{
    VS_Blur_Output OUT;

    OUT = (VS_Blur_Output)0;

    OUT.Pos = float4(IN.Pos.x, IN.Pos.y, 0.5f, 1.0f);

    // Copy all coordinates
    [unroll]
    for( int i = 0; i < MAX_SAMPLES/2; i++ )
        OUT.Tex[i] = IN.Tex[i];

    return OUT;
}

float4 PS_blurBilinear(in VS_Blur_Output IN) : SV_TARGET0
{

    float4 sample = 0.0f;

    for( int i = 0; i < MAX_SAMPLES/2; i++ )
    {
        sample += g_GWeights[2*i + 0] * g_SourceTex.Sample(BilinearSampler, IN.Tex[i].xy);
        sample += g_GWeights[2*i + 1] * g_SourceTex.Sample(BilinearSampler, IN.Tex[i].zw);
    }

    return sample;
}

//=============================================================

technique10 Downscale4x4Bilinear
{
    pass p0
    {
        SetVertexShader  ( CompileShader( vs_4_0, VS_downscale() ) );
        SetGeometryShader( NULL );
        SetPixelShader   ( CompileShader( ps_4_0, PS_downScale2x2() ) );

        SetDepthStencilState( DisableDepth, 0 );
    }
}

technique10 BlurBilinear
{
    pass p0
    {
        SetVertexShader  ( CompileShader( vs_4_0, VS_blur() ) );
        SetGeometryShader( NULL );
        SetPixelShader   ( CompileShader( ps_4_0, PS_blurBilinear() ) );

        SetDepthStencilState( DisableDepth, 0 );
    }
}
//#############################################################################
//
// TONE MAPPING, DOF, LIGHT SCATTERING
//
//#############################################################################

static const float3 LUMINANCE_WEIGHTS = float3(0.27f, 0.67f, 0.06f);
// HDR:
float   g_BlurBlendFactor;
float   g_Exposure;
float   g_FrameDelta;
// DoF:
float focus = 13.0f;
//Light Scattering:
float2 g_ScreenLightPos;
float g_Density = 0.5f;
float g_Weight = 0.9f;
float g_Decay = 0.95f;
float g_RayExposure = 0.85f;
int NUM_SAMPLES = 48;

Texture2D g_HDRBuffer;
Texture2D g_BlurBuffer;
Texture2D g_LumBuffer;
Texture2D g_DofScatteringBuffer;

struct VS_OutputTM
{
    float4 Pos : SV_POSITION;
    float3 Tex : TEXCOORD0;
};

VS_OutputTM VS_toneMapping(in float4 position: POSITION)
{
    VS_OutputTM OUT;

    OUT = (VS_OutputTM)0;

    OUT.Tex.z  = g_Exposure / max(0.001f, g_LumBuffer.SampleLevel(BilinearSampler, float3(0.5f, 0.5f, 0.0f), 0).x);

    OUT.Pos    = float4(position.x, position.y, 0.5f, 1.0f);
    OUT.Tex.xy = position.zw;
    //OUT.Tex.x = 1.0f - OUT.Tex.x;

    return OUT;
}

VS_OutputTM VS_Passthrough(in float4 position: POSITION)
{
    VS_OutputTM OUT;

    OUT = (VS_OutputTM)0;

    OUT.Pos    = float4(position.x, position.y, 0.5f, 1.0f);
    OUT.Tex.xy = position.zw;
    OUT.Tex.z  = 1.0;
    //OUT.Tex.x = 1.0f - OUT.Tex.x;

    return OUT;
}

float3 LightScattering(float2 tex, float3 color)
{
	float2 deltaTexCoord = (tex - g_ScreenLightPos);	
	deltaTexCoord *= (1.0f/NUM_SAMPLES) * g_Density ;
	float illuminationDecay = 1.0f;
	
	for (int i = 0; i<NUM_SAMPLES; i++)
	{
		tex -= deltaTexCoord;
		float3 onesample = g_DofScatteringBuffer.Sample(BilinearSampler, tex).xxx;
		
		onesample *= illuminationDecay * g_Weight;
		color += onesample;
		
		illuminationDecay *= g_Decay;
	}
	
	return color * g_RayExposure;
	
}

float4 PS_toneMapping(in VS_OutputTM In) : SV_TARGET0
{
    float3 c;
    float4 colorDoFScattering = g_DofScatteringBuffer.Sample(BilinearSampler, In.Tex.xy);
    float blendKoeff = min(saturate(abs((colorDoFScattering.y-focus)/20.0f)-0.2f), 0.7f);
    
    //float3 cBlur = float3(1.0f, 0.0f, 0.0f);//g_BlurBuffer.Sample(BilinearSampler, In.Tex.xy).rgb;


    // sum original and blurred image
    //c = (1.0f - g_BlurBlendFactor) * g_HDRBuffer.Sample(BilinearSampler, In.Tex.xy).rgb + g_BlurBlendFactor * g_BlurBuffer.Sample(BilinearSampler, In.Tex.xy).rgb;
    c = lerp(g_HDRBuffer.Sample(BilinearSampler, In.Tex.xy).rgb, g_BlurBuffer.Sample(BilinearSampler, In.Tex.xy).rgb, blendKoeff);

	c += LightScattering(In.Tex.xy, colorDoFScattering.xxx);

    c = c * min( In.Tex.z, 1.7f );
    
    /*
    //**********
    float2 sslp = g_ScreenLightPos;
    sslp.y *= -1.0f;
    sslp *= 0.5f;
    sslp.x += 0.5f;
    sslp.y += 0.5f;
    float c2 = saturate(length(In.Tex.xy - sslp)/0.05f); 
    float3 c3 = c;
    c3.r /= c2;
    return float4(c3, 1.0f);
    //**********
    */

    // exposure
    //c = L / (1 + L);
    //c = L; 

    return float4(c, 1.0);
}

float4 PS_passthrough(in VS_OutputTM In) : SV_TARGET0
{
    half3 c = g_HDRBuffer.Sample(BilinearSampler, In.Tex.xy).rgb;

    return float4(c, 1.0);
}

//===================================================================
technique10 ToneMapping
{
    pass p0
    {
        SetVertexShader  ( CompileShader( vs_4_0, VS_toneMapping() ) );
        SetGeometryShader( NULL );
        SetPixelShader   ( CompileShader( ps_4_0, PS_toneMapping() ) );

        SetDepthStencilState( DisableDepth, 0 );
    }
}
technique10 Passthrough
{
    pass p0
    {
        SetVertexShader  ( CompileShader( vs_4_0, VS_Passthrough() ) );
        SetGeometryShader( NULL );
        SetPixelShader   ( CompileShader( ps_4_0, PS_passthrough() ) );

        SetDepthStencilState( DisableDepth, 0 );
    }
}

//#############################################################################
//
// LUMINANCE
//
//#############################################################################
Texture2D g_CurrentLumBuffer;
Texture2D g_AdaptedLumBuffer;

struct VS_InputLum
{
    float4 Pos    : POSITION;
    float4 Tex[2] : TEXCOORD0;
};

struct VS_OutputLum {
    float4 Pos    : SV_POSITION;
    float4 Tex[2] : TEXCOORD0;
};
//-------------------------------------------------------------
VS_OutputLum VS_luminance(in VS_InputLum IN)
{
    VS_OutputLum OUT;

    OUT = (VS_OutputLum)0;

    OUT.Pos    = float4(IN.Pos.xy, 0.5f, 1.0f);
    OUT.Tex[0] = IN.Tex[0];
    OUT.Tex[1] = IN.Tex[1];

    return OUT;
}
//-------------------------------------------------------------
float4 PS_downscaleLuminanceLog( in VS_OutputLum In ) : SV_TARGET0
{
    half luminance = 0; 

    for( int i = 0; i < 2; i++ )
    {
        luminance += log( dot( g_HDRBuffer.Sample( BilinearSampler, In.Tex[i].xy ).xyz, LUMINANCE_WEIGHTS ) + 0.0001f);
        luminance += log( dot( g_HDRBuffer.Sample( BilinearSampler, In.Tex[i].zw ).xyz, LUMINANCE_WEIGHTS ) + 0.0001f);
    }

    luminance *= 0.25f;

    return float4(luminance, luminance, luminance, 1.0);
}
//-------------------------------------------------------------
float4 PS_downscaleLuminance( in VS_OutputLum In ) : SV_TARGET0
{
    half sample = 0.0f;

    for( int i = 0; i < 2; i++ )
    {
        sample += g_LumBuffer.Sample( BilinearSampler, In.Tex[i].xy ).x;
        sample += g_LumBuffer.Sample( BilinearSampler, In.Tex[i].zw ).x;
    }

    sample *= 0.25f;

    return float4(sample, sample, sample, 1.0);

}
//-------------------------------------------------------------
float4 PS_downscaleLuminanceExp( in VS_OutputLum In ) : SV_TARGET0
{
    half sample = 0.0f;

    for( int i = 0; i < 2; i++ )
    {
        sample += g_LumBuffer.Sample( BilinearSampler, In.Tex[i].xy ).x;
        sample += g_LumBuffer.Sample( BilinearSampler, In.Tex[i].zw ).x;
    }

    sample *= 0.25f;

    return exp(sample);
}
//-------------------------------------------------------------
float4 VS_PassthroughLum(in float4 position: POSITION) : SV_POSITION
{
    float4 OUT;

    OUT = (float4)0;

    return float4(position.x, position.y, 0.5f, 1.0f);
}
//-------------------------------------------------------------
float4 PS_LuminanceAdaptation(in float4 position: SV_POSITION) : SV_TARGET0
{
    half adaptedLum = g_AdaptedLumBuffer.Sample(BilinearSampler, float2(0.5f, 0.5f));
    half currentLum = g_CurrentLumBuffer.Sample(BilinearSampler, float2(0.5f, 0.5f));

    return (adaptedLum + (currentLum - adaptedLum) * ( 1 - pow( 0.98f, 300 * g_FrameDelta ) ));
}

//=============================================================
technique10 DownscaleLuminanceLog
{
    pass p0
    {
        SetVertexShader  ( CompileShader( vs_4_0, VS_luminance() ) );
        SetGeometryShader( NULL );
        SetPixelShader   ( CompileShader( ps_4_0, PS_downscaleLuminanceLog() ) );

        SetDepthStencilState( DisableDepth, 0 );
    }
}

//=============================================================
technique10 DownscaleLuminance
{
    pass p0
    {
        SetVertexShader  ( CompileShader( vs_4_0, VS_luminance() ) );
        SetGeometryShader( NULL );
        SetPixelShader   ( CompileShader( ps_4_0, PS_downscaleLuminance() ) );

        SetDepthStencilState( DisableDepth, 0 );
    }
}

//=============================================================
technique10 DownscaleLuminanceExp
{
    pass p0
    {
        SetVertexShader  ( CompileShader( vs_4_0, VS_luminance() ) );
        SetGeometryShader( NULL );
        SetPixelShader   ( CompileShader( ps_4_0, PS_downscaleLuminanceExp() ) );

        SetDepthStencilState( DisableDepth, 0 );
    }
}

//=============================================================
technique10 LuminanceAdaptation
{
    pass p0
    {
        SetVertexShader  ( CompileShader( vs_4_0, VS_PassthroughLum() ) );
        SetGeometryShader( NULL );
        SetPixelShader   ( CompileShader( ps_4_0, PS_LuminanceAdaptation() ) );

        SetDepthStencilState( DisableDepth, 0 );
    }
}



